C     =================================================================
C     File: ballswithinellipse.f
C     =================================================================

C     =================================================================
C     Module: Subroutines that define the problem
C     =================================================================

C     Last update of any of the component of this module:

C     June 1, 2012.

C     Users are encouraged to download periodically updated versions of
C     this code at the TANGO home page:

C     www.ime.usp.br/~egbirgin/tango/

C     ******************************************************************
C     ******************************************************************

      subroutine inip(n,x,l,u,m,lambda,equatn,linear,coded,checkder)

      implicit none

C     SCALAR ARGUMENTS
      integer m,n
      logical checkder

C     ARRAY ARGUMENTS
      logical coded(10),equatn(*),linear(*)
      double precision l(*),lambda(*),u(*),x(*)

C     LOCAL SCALARS
      integer i,j,k
      double precision seed

C     PARAMETERS
      integer nballsmax,npairsmax
      parameter ( nballsmax = 100 )
      parameter ( npairsmax = nballsmax * ( nballsmax - 1 ) / 2 )

C     COMMON SCALARS
      integer nballs
      double precision a,b,r

C     COMMON ARRAYS
      integer pairs(2,npairsmax)

C     COMMON BLOCKS
      common /elldata/ pairs,a,b,r,nballs

C     EXTERNAL FUNCTIONS
      integer P
      double precision drand
      external drand,P

C     Problem data

      a = 4.0d0
      b = 2.0d0
      r = 1.0d0
      nballs = 3

C     Variables

      n = 3 * nballs

      do i = 1,nballs
          l(P('u',i)) = - a
          u(P('u',i)) =   a
          l(P('v',i)) = - b
          u(P('v',i)) =   b
          l(P('s',i)) = 0.0d0
          u(P('s',i)) = 1.0d0
      end do

      write(*,*) 'Enter seed: '
      read(*,*) seed
c     seed = 123456.0d0

      do i = 1,n
          x(i) = l(i) + ( u(i) - l(i) ) * drand(seed)
      end do

C     Upper-level Constraints

      m = 2 * nballs + nballs * ( nballs - 1 ) / 2

      k = 0
      do i = 1,nballs
          do j = i + 1,nballs
              k = k + 1
              pairs(1,k) = i
              pairs(2,k) = j
          end do
      end do

      do i = 1,m
          linear(i) = .false.
          lambda(i) =  0.0d0
      end do

      do i = 1,nballs
          equatn(i) = .true.
      end do

      do i = nballs + 1,2 * nballs
          equatn(i) = .false.
      end do

      do i = 2 * nballs + 1,2 * nballs + nballs * ( nballs - 1 ) / 2
          equatn(i) = .false.
      end do

C     Indicate which subroutines did you code.

      coded( 1) = .true.  ! evalf
      coded( 2) = .true.  ! evalg
      coded( 3) = .true.  ! evalh
      coded( 4) = .true.  ! evalc
      coded( 5) = .true.  ! evaljac
      coded( 6) = .true.  ! evalhc
      coded( 7) = .false. ! evalfc
      coded( 8) = .false. ! evalgjac
      coded( 9) = .false. ! evalhl
      coded(10) = .false. ! evalhlp

C     Set checkder = TRUE if you code some derivatives and you would
C     like them to be tested by finite differences. It is highly
C     recommended.

      checkder = .false.

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalf(n,x,f,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer flag,n
      double precision f

C     ARRAY ARGUMENTS
      double precision x(n)

C     PARAMETERS
      integer nballsmax,npairsmax
      parameter ( nballsmax = 100 )
      parameter ( npairsmax = nballsmax * ( nballsmax - 1 ) / 2 )

C     COMMON SCALARS
      integer nballs
      double precision a,b,r

C     COMMON ARRAYS
      integer pairs(2,npairsmax)

C     LOCAL SCALARS
      integer i,j
      double precision cte,dist,ui,uj,vi,vj,si,sj,xxi,xxj,yyi,yyj

C     COMMON BLOCKS
      common /elldata/ pairs,a,b,r,nballs

C     EXTERNAL FUNCTIONS
      integer P
      external P

      flag = 0

      f = 0.0d0

      cte = b ** 2 / a ** 2

      do i = 1,nballs
          do j = i + 1,nballs
              ui = x(P('u',i))
              vi = x(P('v',i))
              si = x(P('s',i))

              xxi = ( 1.0d0 + ( si - 1.0d0 ) * cte ) * ui
              yyi = si * vi

              uj = x(P('u',j))
              vj = x(P('v',j))
              sj = x(P('s',j))

              xxj = ( 1.0d0 + ( sj - 1.0d0 ) * cte ) * uj
              yyj = sj * vj

              dist = ( xxi - xxj ) ** 2 + ( yyi - yyj ) ** 2
              f = f - dist
          end do
      end do              

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalg(n,x,g,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer flag,n

C     ARRAY ARGUMENTS
      double precision g(n),x(n)

C     PARAMETERS
      integer nballsmax,npairsmax
      parameter ( nballsmax = 100 )
      parameter ( npairsmax = nballsmax * ( nballsmax - 1 ) / 2 )

C     COMMON SCALARS
      integer nballs
      double precision a,b,r

C     COMMON ARRAYS
      integer pairs(2,npairsmax)

C     LOCAL SCALARS
      integer i,j
      double precision cte,diffx,diffy,ui,uj,vi,vj,si,sj,xxi,xxj,yyi,yyj

C     COMMON BLOCKS
      common /elldata/ pairs,a,b,r,nballs

C     EXTERNAL FUNCTIONS
      integer P
      external P

      flag = 0

      do i = 1,nballs
          g(P('u',i)) = 0.0d0
          g(P('v',i)) = 0.0d0
          g(P('s',i)) = 0.0d0
      end do              

      cte = b ** 2 / a ** 2

      do i = 1,nballs
          do j = i + 1,nballs
              ui = x(P('u',i))
              vi = x(P('v',i))
              si = x(P('s',i))

              xxi = ( 1.0d0 + ( si - 1.0d0 ) * cte ) * ui
              yyi = si * vi

              uj = x(P('u',j))
              vj = x(P('v',j))
              sj = x(P('s',j))

              xxj = ( 1.0d0 + ( sj - 1.0d0 ) * cte ) * uj
              yyj = sj * vj

              diffx = 2.0d0 * ( xxi - xxj )
              diffy = 2.0d0 * ( yyi - yyj )

              g(P('u',i)) = 
     +        g(P('u',i)) - diffx * ( 1.0d0 + ( si - 1.0d0 ) * cte )
              g(P('u',j)) = 
     +        g(P('u',j)) + diffx * ( 1.0d0 + ( sj - 1.0d0 ) * cte )

              g(P('v',i)) = g(P('v',i)) - diffy * si
              g(P('v',j)) = g(P('v',j)) + diffy * sj

              g(P('s',i)) = g(P('s',i)) - diffx * cte * ui - diffy * vi
              g(P('s',j)) = g(P('s',j)) + diffx * cte * uj + diffy * vj
          end do
      end do              

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalh(n,x,hlin,hcol,hval,hnnz,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer flag,n,hnnz

C     ARRAY ARGUMENTS
      integer hcol(*),hlin(*)
      double precision hval(*),x(n)

C     PARAMETERS
      integer nballsmax,npairsmax
      parameter ( nballsmax = 100 )
      parameter ( npairsmax = nballsmax * ( nballsmax - 1 ) / 2 )

C     COMMON SCALARS
      integer nballs
      double precision a,b,r

C     COMMON ARRAYS
      integer pairs(2,npairsmax)

C     LOCAL SCALARS
      integer i,j
      double precision cte,diffx,diffy,dxxidui,dxxjduj,ui,uj,vi,vj,si,
     +        sj,xxi,xxj,yyi,yyj

C     COMMON BLOCKS
      common /elldata/ pairs,a,b,r,nballs

C     EXTERNAL FUNCTIONS
      integer P
      external P

      flag = 0

      cte = b ** 2 / a ** 2

      hnnz = 0

      do i = 1,nballs
          do j = i + 1,nballs
              ui = x(P('u',i))
              vi = x(P('v',i))
              si = x(P('s',i))

              xxi = ( 1.0d0 + ( si - 1.0d0 ) * cte ) * ui
              yyi = si * vi

              uj = x(P('u',j))
              vj = x(P('v',j))
              sj = x(P('s',j))

              xxj = ( 1.0d0 + ( sj - 1.0d0 ) * cte ) * uj
              yyj = sj * vj

              diffx = 2.0d0 * ( xxi - xxj )
              diffy = 2.0d0 * ( yyi - yyj )

              dxxidui = ( 1.0d0 + ( si - 1.0d0 ) * cte )
              dxxjduj = ( 1.0d0 + ( sj - 1.0d0 ) * cte )

              hlin(hnnz+1) = P('u',i)
              hcol(hnnz+1) = P('u',i)
              hval(hnnz+1) = - 2.0d0 * dxxidui ** 2

              hlin(hnnz+2) = P('s',i)
              hcol(hnnz+2) = P('u',i)
              hval(hnnz+2) = - 2.0d0 * cte * ui * dxxidui - diffx * cte

              hlin(hnnz+3) = P('u',j)
              hcol(hnnz+3) = P('u',i)
              hval(hnnz+3) = 2.0d0 * dxxjduj * dxxidui

              hlin(hnnz+4) = P('s',j)
              hcol(hnnz+4) = P('u',i)
              hval(hnnz+4) = 2.0d0 * cte * uj * dxxidui 

              hlin(hnnz+5) = P('v',i)
              hcol(hnnz+5) = P('v',i)
              hval(hnnz+5) = - 2.0d0 * si ** 2

              hlin(hnnz+6) = P('s',i)
              hcol(hnnz+6) = P('v',i)
              hval(hnnz+6) = - 4.0d0 * yyi +  2.0d0 * yyj

              hlin(hnnz+7) = P('v',j)
              hcol(hnnz+7) = P('v',i)
              hval(hnnz+7) = 2.0d0 * sj * si

              hlin(hnnz+8) = P('s',j)
              hcol(hnnz+8) = P('v',i)
              hval(hnnz+8) = 2.0d0 * vj * si

              hlin(hnnz+9) = P('s',i)
              hcol(hnnz+9) = P('s',i)
              hval(hnnz+9) = - 2.0d0 * ( cte ** 2 * ui ** 2 + vi ** 2 )

              hlin(hnnz+10) = P('s',j)
              hcol(hnnz+10) = P('s',i)
              hval(hnnz+10) = 2.0d0 * ( cte ** 2 * ui * uj + vj * vi )

              hlin(hnnz+11) = P('s',i)
              hcol(hnnz+11) = P('u',j)
              hval(hnnz+11) = 2.0d0 * cte * ui * dxxjduj

              hlin(hnnz+12) = P('u',j)
              hcol(hnnz+12) = P('u',j)
              hval(hnnz+12) = - 2.0d0 * dxxjduj ** 2 

              hlin(hnnz+13) = P('s',j)
              hcol(hnnz+13) = P('u',j)
              hval(hnnz+13) = - 2.0d0 * cte * uj * dxxjduj + diffx * cte

              hlin(hnnz+14) = P('s',i)
              hcol(hnnz+14) = P('v',j)
              hval(hnnz+14) = 2.0d0 * vi * sj

              hlin(hnnz+15) = P('v',j)
              hcol(hnnz+15) = P('v',j)
              hval(hnnz+15) = - 2.0d0 * sj ** 2

              hlin(hnnz+16) = P('s',j)
              hcol(hnnz+16) = P('v',j)
              hval(hnnz+16) = 2.0d0 * yyi - 4.0d0 * yyj

              hlin(hnnz+17) = P('s',j)
              hcol(hnnz+17) = P('s',j)
              hval(hnnz+17) = - 2.0d0 * ( cte ** 2 * uj ** 2 + vj ** 2 )

              hnnz = hnnz + 17
          end do
      end do              

      end

C     ******************************************************************
C     ******************************************************************
 
      subroutine evalc(n,x,ind,c,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer ind,flag,n
      double precision c

C     ARRAY ARGUMENTS
      double precision x(n)

C     LOCAL SCALARS
      integer i,j,k
      double precision cte,ui,uj,vi,vj,si,sj,xxi,xxj,yyi,yyj

C     PARAMETERS
      integer nballsmax,npairsmax
      parameter ( nballsmax = 100 )
      parameter ( npairsmax = nballsmax * ( nballsmax - 1 ) / 2 )

C     COMMON SCALARS
      integer nballs
      double precision a,b,r

C     COMMON ARRAYS
      integer pairs(2,npairsmax)

C     COMMON BLOCKS
      common /elldata/ pairs,a,b,r,nballs

C     EXTERNAL FUNCTIONS
      integer P
      external P

      flag = 0

      if ( ind .le. nballs ) then
          i  = ind

          ui = x(P('u',i))
          vi = x(P('v',i))

          c = ( ui / a ) ** 2 + ( vi / b ) ** 2 - 1.0d0

      else if ( ind .le. 2 * nballs ) then
          i  = ind - nballs

          cte = ( b ** 2 / a ** 2 ) ** 2

          ui = x(P('u',i))
          vi = x(P('v',i))
          si = x(P('s',i))

          c = r ** 2 - ( si - 1.0d0 ) ** 2 * ( cte * ui ** 2 + vi ** 2 )

      else
          k = ind - 2 * nballs
          i = pairs(1,k)
          j = pairs(2,k)

          cte = b ** 2 / a ** 2

          ui = x(P('u',i))
          vi = x(P('v',i))
          si = x(P('s',i))

          xxi = ( 1.0d0 + ( si - 1.0d0 ) * cte ) * ui
          yyi = si * vi

          uj = x(P('u',j))
          vj = x(P('v',j))
          sj = x(P('s',j))

          xxj = ( 1.0d0 + ( sj - 1.0d0 ) * cte ) * uj
          yyj = sj * vj

          c = 4 * r ** 2 - ( xxi - xxj ) ** 2 - ( yyi - yyj ) ** 2
      end if

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evaljac(n,x,ind,jcvar,jcval,jcnnz,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer flag,ind,jcnnz,n

C     ARRAY ARGUMENTS
      integer jcvar(n)
      double precision x(n),jcval(n)

C     LOCAL SCALARS
      integer i,j,k
      double precision cte,diffx,diffy,ui,uj,vi,vj,si,sj,xxi,xxj,yyi,yyj

C     PARAMETERS
      integer nballsmax,npairsmax
      parameter ( nballsmax = 100 )
      parameter ( npairsmax = nballsmax * ( nballsmax - 1 ) / 2 )

C     COMMON SCALARS
      integer nballs
      double precision a,b,r

C     COMMON ARRAYS
      integer pairs(2,npairsmax)

C     COMMON BLOCKS
      common /elldata/ pairs,a,b,r,nballs

C     EXTERNAL FUNCTIONS
      integer P
      external P

      flag = 0

      if ( ind .le. nballs ) then
          i  = ind

          ui = x(P('u',i))
          vi = x(P('v',i))

          jcnnz = 2

          jcvar(1) = P('u',i)
          jcval(1) = 2.0d0 * ui / a ** 2

          jcvar(2) = P('v',i)
          jcval(2) = 2.0d0 * vi / b ** 2

      else if ( ind .le. 2 * nballs ) then
          i  = ind - nballs

          cte = ( b ** 2 / a ** 2 ) ** 2

          ui = x(P('u',i))
          vi = x(P('v',i))
          si = x(P('s',i))

          jcnnz = 3

          jcvar(1) = P('u',i)
          jcval(1) = - ( si - 1.0d0 ) ** 2 * 2.0d0 * cte * ui

          jcvar(2) = P('v',i)
          jcval(2) = - ( si - 1.0d0 ) ** 2 * 2.0d0 * vi

          jcvar(3) = P('s',i)
          jcval(3) = - 2.0d0 * ( si - 1.0d0 ) * 
     +                 ( cte * ui ** 2 + vi ** 2 )

      else
          k = ind - 2 * nballs
          i = pairs(1,k)
          j = pairs(2,k)

          cte = b ** 2 / a ** 2

          ui = x(P('u',i))
          vi = x(P('v',i))
          si = x(P('s',i))

          xxi = ( 1.0d0 + ( si - 1.0d0 ) * cte ) * ui
          yyi = si * vi

          uj = x(P('u',j))
          vj = x(P('v',j))
          sj = x(P('s',j))

          xxj = ( 1.0d0 + ( sj - 1.0d0 ) * cte ) * uj
          yyj = sj * vj

          diffx = 2.0d0 * ( xxi - xxj )
          diffy = 2.0d0 * ( yyi - yyj )

          jcnnz = 6

          jcvar(1) = P('u',i)
          jcval(1) = - diffx * ( 1.0d0 + ( si - 1.0d0 ) * cte )

          jcvar(2) = P('v',i)
          jcval(2) = - diffy * si

          jcvar(3) = P('s',i)
          jcval(3) = - diffx * cte * ui - diffy * vi

          jcvar(4) = P('u',j)
          jcval(4) = diffx * ( 1.0d0 + ( sj - 1.0d0 ) * cte )

          jcvar(5) = P('v',j)
          jcval(5) = diffy * sj

          jcvar(6) = P('s',j)
          jcval(6) = diffx * cte * uj + diffy * vj
      end if

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalhc(n,x,ind,hclin,hccol,hcval,hcnnz,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer flag,hcnnz,ind,n

C     ARRAY ARGUMENTS
      integer hccol(*),hclin(*)
      double precision hcval(*),x(n)

C     LOCAL SCALARS
      integer i,j,k
      double precision cte,diffx,diffy,dxxidui,dxxjduj,ui,uj,vi,vj,si,
     +        sj,xxi,xxj,yyi,yyj

C     PARAMETERS
      integer nballsmax,npairsmax
      parameter ( nballsmax = 100 )
      parameter ( npairsmax = nballsmax * ( nballsmax - 1 ) / 2 )

C     COMMON SCALARS
      integer nballs
      double precision a,b,r

C     COMMON ARRAYS
      integer pairs(2,npairsmax)

C     COMMON BLOCKS
      common /elldata/ pairs,a,b,r,nballs

C     EXTERNAL FUNCTIONS
      integer P
      external P

      flag = 0

      if ( ind .le. nballs ) then
          i  = ind

          ui = x(P('u',i))
          vi = x(P('v',i))

          hcnnz = 2

          hclin(1) = P('u',i)
          hccol(1) = P('u',i)
          hcval(1) = 2.0d0 / a ** 2

          hclin(2) = P('v',i)
          hccol(2) = P('v',i)
          hcval(2) = 2.0d0 / b ** 2

      else if ( ind .le. 2 * nballs ) then
          i  = ind - nballs

          cte = ( b ** 2 / a ** 2 ) ** 2

          ui = x(P('u',i))
          vi = x(P('v',i))
          si = x(P('s',i))

          hcnnz = 5

          hclin(1) = P('u',i)
          hccol(1) = P('u',i)
          hcval(1) = - ( si - 1.0d0 ) ** 2 * 2.0d0 * cte

          hclin(2) = P('s',i)
          hccol(2) = P('u',i)
          hcval(2) = - 4.0d0 * ( si - 1.0d0 ) * cte * ui

          hclin(3) = P('v',i)
          hccol(3) = P('v',i)
          hcval(3) = - ( si - 1.0d0 ) ** 2 * 2.0d0

          hclin(4) = P('s',i)
          hccol(4) = P('v',i)
          hcval(4) = - 4.0d0 * ( si - 1.0d0 ) * vi

          hclin(5) = P('s',i)
          hccol(5) = P('s',i)
          hcval(5) = - 2.0d0 * ( cte * ui ** 2 + vi ** 2 )

      else
          k = ind - 2 * nballs
          i = pairs(1,k)
          j = pairs(2,k)

          cte = b ** 2 / a ** 2

          ui = x(P('u',i))
          vi = x(P('v',i))
          si = x(P('s',i))

          xxi = ( 1.0d0 + ( si - 1.0d0 ) * cte ) * ui
          yyi = si * vi

          uj = x(P('u',j))
          vj = x(P('v',j))
          sj = x(P('s',j))

          xxj = ( 1.0d0 + ( sj - 1.0d0 ) * cte ) * uj
          yyj = sj * vj

          diffx = 2.0d0 * ( xxi - xxj )
          diffy = 2.0d0 * ( yyi - yyj )

          dxxidui = ( 1.0d0 + ( si - 1.0d0 ) * cte )
          dxxjduj = ( 1.0d0 + ( sj - 1.0d0 ) * cte )

          hcnnz = 17

          hclin(1) = P('u',i)
          hccol(1) = P('u',i)
          hcval(1) = - 2.0d0 * dxxidui ** 2

          hclin(2) = P('s',i)
          hccol(2) = P('u',i)
          hcval(2) = - 2.0d0 * cte * ui * dxxidui - diffx * cte

          hclin(3) = P('u',j)
          hccol(3) = P('u',i)
          hcval(3) = 2.0d0 * dxxjduj * dxxidui

          hclin(4) = P('s',j)
          hccol(4) = P('u',i)
          hcval(4) = 2.0d0 * cte * uj * dxxidui 

          hclin(5) = P('v',i)
          hccol(5) = P('v',i)
          hcval(5) = - 2.0d0 * si ** 2

          hclin(6) = P('s',i)
          hccol(6) = P('v',i)
          hcval(6) = - 4.0d0 * yyi +  2.0d0 * yyj

          hclin(7) = P('v',j)
          hccol(7) = P('v',i)
          hcval(7) = 2.0d0 * sj * si

          hclin(8) = P('s',j)
          hccol(8) = P('v',i)
          hcval(8) = 2.0d0 * vj * si

          hclin(9) = P('s',i)
          hccol(9) = P('s',i)
          hcval(9) = - 2.0d0 * ( cte ** 2 * ui ** 2 + vi ** 2 )

          hclin(10) = P('s',j)
          hccol(10) = P('s',i)
          hcval(10) = 2.0d0 * ( cte ** 2 * ui * uj + vj * vi )

          hclin(11) = P('s',i)
          hccol(11) = P('u',j)
          hcval(11) = 2.0d0 * cte * ui * dxxjduj

          hclin(12) = P('u',j)
          hccol(12) = P('u',j)
          hcval(12) = - 2.0d0 * dxxjduj ** 2 

          hclin(13) = P('s',j)
          hccol(13) = P('u',j)
          hcval(13) = - 2.0d0 * cte * uj * dxxjduj + diffx * cte

          hclin(14) = P('s',i)
          hccol(14) = P('v',j)
          hcval(14) = 2.0d0 * vi * sj

          hclin(15) = P('v',j)
          hccol(15) = P('v',j)
          hcval(15) = - 2.0d0 * sj ** 2

          hclin(16) = P('s',j)
          hccol(16) = P('v',j)
          hcval(16) = 2.0d0 * yyi - 4.0d0 * yyj

          hclin(17) = P('s',j)
          hccol(17) = P('s',j)
          hcval(17) = - 2.0d0 * ( cte ** 2 * uj ** 2 + vj ** 2 )
      end if

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalfc(n,x,f,m,c,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer flag,m,n
      double precision f

C     ARRAY ARGUMENTS
      double precision c(m),x(n)

      flag = - 1

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalgjac(n,x,g,m,jcfun,jcvar,jcval,jcnnz,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer flag,jcnnz,m,n

C     ARRAY ARGUMENTS
      integer jcfun(*),jcvar(*)
      double precision g(n),jcval(*),x(n)

      flag = - 1

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalhl(n,x,m,lambda,sf,sc,hllin,hlcol,hlval,hlnnz,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer flag,hlnnz,m,n
      double precision sf

C     ARRAY ARGUMENTS
      integer hlcol(*),hllin(*)
      double precision hlval(*),lambda(m),sc(m),x(n)

      flag = - 1

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalhlp(n,x,m,lambda,sf,sc,p,hp,goth,flag)

      implicit none

C     SCALAR ARGUMENTS
      logical goth
      integer flag,m,n
      double precision sf

C     ARRAY ARGUMENTS
      double precision hp(n),lambda(m),p(n),sc(m),x(n)

      flag = - 1

      end

C     ******************************************************************
C     ******************************************************************

      subroutine endp(n,x,l,u,m,lambda,equatn,linear)

      implicit none

C     SCALAR ARGUMENTS
      integer m,n

C     ARRAY ARGUMENTS
      logical equatn(m),linear(m)
      double precision l(n),lambda(m),u(n),x(n)

      end

C     ******************************************************************
C     ******************************************************************
 
      integer function P(vtype,vind)

      implicit none

C     SCALAR ARGUMENTS
      integer vind
      character vtype

C     PARAMETERS
      integer nballsmax,npairsmax
      parameter ( nballsmax = 100 )
      parameter ( npairsmax = nballsmax * ( nballsmax - 1 ) / 2 )

C     COMMON SCALARS
      integer nballs
      double precision a,b,r

C     COMMON ARRAYS
      integer pairs(2,npairsmax)

C     LOCAL SCALARS
      integer i,j
      double precision cte,dist,ui,uj,vi,vj,si,sj,xxi,xxj,yyi,yyj

C     COMMON BLOCKS
      common /elldata/ pairs,a,b,r,nballs

      if ( vtype .eq. 'u' ) then
          P = vind
      else if ( vtype .eq. 'v' ) then
          P = vind + nballs
      else if ( vtype .eq. 's' ) then
          P = vind + 2 * nballs
      else
          write(*,*) 'Error in L function!'
          stop
      end if

      return

      end

