C     =================================================================
C     File: cs.f
C     =================================================================

C     =================================================================
C     Module: Subroutines that define the problem
C     =================================================================

C     Last update of any of the component of this module:

C     June 29, 2012.

C     Users are encouraged to download periodically updated versions of
C     this code at the TANGO home page:

C     www.ime.usp.br/~egbirgin/tango/

C     ******************************************************************
C     ******************************************************************

      subroutine inip(n,x,l,u,m,lambda,equatn,linear,coded,checkder)

      implicit none

C     SCALAR ARGUMENTS
      integer m,n
      logical checkder

C     ARRAY ARGUMENTS
      logical coded(11),equatn(*),linear(*)
      double precision l(*),lambda(*),u(*),x(*)

#include "../algencan/dim.par"
#include "cs.com"

C     LOCAL SCALARS
      integer i,j,sl0norm,pij
      double precision seed,sl1norm

C     LOCAL ARRAYS
      double precision s(nsignmax),s0(nsignmax),tmp(msamplmax),
     +        tmp2(nsignmax)
      character * 80 fname

C     EXTERNAL FUNCTIONS
      double precision drand
      external drand

      if ( 2 * nsignmax .gt. nmax ) then
          write(*,*) 'Increase nmax or reduce nsignmax'
          stop
      end if

C     Read problem data 
   
      write(*,*) 'Enter true signal input file name: '
      read(*,*) fname
      
      open(20,file=fname)

      read(20,*) nsignrows,nsigncols

      nsign = nsignrows * nsigncols

      if ( nsign .gt. nsignmax ) then
          write(*,*) 'ERROR: Increase nsignmax to at least ',nsign
          stop
      end if

      l2nsign = 0
 10   if ( 2 ** l2nsign .lt. nsign ) then
          l2nsign = l2nsign + 1
          go to 10
      end if

      if ( 2 ** l2nsign .ne. nsign ) then
          write(*,*)  'ERROR: nsign must be a power of two.'
          stop
      end if

      do i = 1,nsignrows
          read(20,*) (truesignal((i-1)*nsigncols+j),j=1,nsigncols)
      end do

      close(20)

      write(*,*) 'Enter number of desired samples: '
      read(*,*) msampl

      if ( msampl .gt. msamplmax ) then
          write(*,*) 'ERROR: Increase msamplmax to at least ',msampl
          stop
      end if

      if ( msampl .gt. nsign ) then
          write(*,*) 'ERROR: msampl must satisfy msampl < nsign'
          stop
      end if

      write(*,*) 'Enter L parameter for sampling Phi matrix: '
      read(*,*) lPhi

      if ( lPhi .gt. lPhimax ) then
          write(*,*) 'ERROR: Increase lPhimax to at least ',lPhi
          stop
      end if

      if ( lPhi .gt. msampl ) then
          write(*,*) 'ERROR: lPhi must satisfy lPhi < msampl'
          stop
      end if
      
C     Generate permutation matrices for sampling matrix Phi
      
      seed = 123456.0d0

      do j = 1,lPhi
          do i = 1,nsign
              pij    = floor( ( nsign - i + 1 ) * drand(seed) )
              P(i,j) = i + min( pij, nsign - i )
          end do
      end do

C     Sampling y from the true signal

      call multphi(0,truesignal,y)

      open(10,file="samples.dat")

      do i = 1,msampl
          write(10,*) y(i)
      end do

      close(10)
     
C     Compute initial guess x = Phi^t (Phi Phi^t)^{-1} y

      call CGrad(tmp)
      call multphi(1,tmp,tmp2)
      call multpsi(0,tmp2,x)
      
C     Compute known solution s for comparison purposes only (s are the 
C     coefficients of the true signal as a linear combination of the 
C     rows of Psi)

      call multpsi(0,truesignal,s)

      sl0norm = 0
      sl1norm = 0.0d0
      do i = 1,nsign
          if ( s(i) .ne. 0.0d0 ) then
              sl0norm = sl0norm + 1
          end if
          sl1norm = sl1norm + abs( s(i) )
      end do

      write(*,*) 'Known solution l0-norm =',float(sl0norm) / nsign
      write(*,*) 'Known solution l1-norm =',sl1norm

C     Show problem information

      write(*,*) 'Problem generated using signal file: ',fname
      write(*,*) 'nsignrows: ',nsignrows
      write(*,*) 'nsigncols: ',nsigncols
      write(*,*) 'nsign    : ',nsign
      write(*,*) 'msampl   : ',msampl
      write(*,*) 'lPhi     : ',lPhi

C     Number of variables

      n =  2 * nsign

C     Initial point

      do i = 1,nsign
          if ( s0(i) .ge. 0.0d0 ) then
              x(i)         =   s0(i)
              x(nsign + i) =   0.0d0
          else
              x(i)         =   0.0d0
              x(nsign + i) = - s0(i)
          end if
      end do

C     Lower and upper bounds

      do i = 1,n
          l(i) = 0.0d0
          u(i) = 1.0d+20
      end do

C     Number of constraints (equalities plus inequalities)

      m = msampl

C     Lagrange multipliers approximation.

      do i = 1,m
          lambda(i) =  0.0d0
      end do

C     For each constraint i, set equatn(i) = .true. if it is an equality
C     constraint of the form c_i(x) = 0, and set equatn(i) = .false. if
C     it is an inequality constraint of the form c_i(x) <= 0.

      do i = 1,m
          equatn(i) = .true.
      end do

C     For each constraint i, set linear(i) = .true. if it is a linear
C     constraint, otherwise set linear(i) = .false.

      do i = 1,m
          linear(i) = .true.
      end do

C     Indicate which subroutines did you code.

      coded( 1) = .false. ! evalf
      coded( 2) = .false. ! evalg
      coded( 3) = .false. ! evalh
      coded( 4) = .false. ! evalc
      coded( 5) = .false. ! evaljac
      coded( 6) = .false. ! evalhc
      coded( 7) = .true.  ! evalfc
      coded( 8) = .false. ! evalgjac
      coded( 9) = .true.  ! evalgjacp
      coded(10) = .false. ! evalhl
      coded(11) = .true.  ! evalhlp

C     Set checkder = TRUE if you code some derivatives and you would
C     like them to be tested by finite differences. It is highly
C     recommended.

      checkder = .false.

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalf(n,x,f,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer flag,n
      double precision f

C     ARRAY ARGUMENTS
      double precision x(n)

      flag = - 1

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalg(n,x,g,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer flag,n

C     ARRAY ARGUMENTS
      double precision g(n),x(n)

      flag = - 1

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalh(n,x,hlin,hcol,hval,hnnz,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer flag,n,hnnz

C     ARRAY ARGUMENTS
      integer hcol(*),hlin(*)
      double precision hval(*),x(n)

      flag = - 1

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalc(n,x,ind,c,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer ind,flag,n
      double precision c

C     ARRAY ARGUMENTS
      double precision x(n)

      flag = - 1

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evaljac(n,x,ind,jcvar,jcval,jcnnz,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer flag,ind,jcnnz,n

C     ARRAY ARGUMENTS
      integer jcvar(n)
      double precision x(n),jcval(n)

      flag = - 1

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalhc(n,x,ind,hclin,hccol,hcval,hcnnz,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer flag,hcnnz,ind,n

C     ARRAY ARGUMENTS
      integer hccol(*),hclin(*)
      double precision hcval(*),x(n)

      flag = - 1

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalfc(n,x,f,m,c,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer flag,m,n
      double precision f

C     ARRAY ARGUMENTS
      double precision c(m),x(n)

#include "cs.com"

C     LOCAL SCALARS
      integer i

C     LOCAL ARRAYS
      double precision tmp(nsignmax),uv(nsignmax)

      flag = 0

C     Objective function

      f = 0.0d0
      do i = 1,n
          f = f + x(i)
      end do

C     Constraints

      do i = 1,nsign
          uv(i) = x(i) - x(i + nsign)
      end do

      call multpsi(1,uv,tmp)
      call multphi(0,tmp,c)

      do i = 1,msampl
          c(i) = c(i) - y(i)
      end do

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalgjac(n,x,g,m,jcfun,jcvar,jcval,jcnnz,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer flag,jcnnz,m,n

C     ARRAY ARGUMENTS
      integer jcfun(*),jcvar(*)
      double precision g(n),jcval(*),x(n)

      flag = - 1

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalgjacp(n,x,g,m,pp,q,work,gotj,flag)

      implicit none

C     SCALAR ARGUMENTS
      logical gotj
      integer flag,m,n
      character work

C     ARRAY ARGUMENTS
      double precision g(n),pp(m),q(n),x(n)

#include "cs.com"

C     LOCAL SCALARS
      integer i

C     LOCAL ARRAYS
      double precision tmp(nsignmax),uv(nsignmax)

C     The meaning of argument work follows: work = 'j' or 'J' means that 
C     q is an input array argument and that p = Jacobian x q must be 
C     computed, while work = 't' or 'T' means that p is an input array 
C     argument and that q = Jacobian^t x p must be computed. Moreover, a 
C     capital letter (i.e. 'J' or 'T') means that the gradient of the 
C     objective function g must also be computed. A lower letter (i.e. 
C     'j' or 't') means that only the product with the Jacobian is 
C     required. In the later case, input array argument g MUST NOT be 
C     modified nor referenced.

      flag = 0

C     Gradient of the objective function

      if ( work .eq. 'J' .or. work .eq. 'T' ) then
          do i = 1,n
              g(i) = 1.0d0
          end do
      end if

C     p = Jacobian x q
      if ( work .eq. 'j' .or. work .eq. 'J' ) then
          do i = 1,nsign
              uv(i) = q(i) - q(i + nsign)
          end do

          call multpsi(1,uv,tmp)
          call multphi(0,tmp,pp)

C     q = Jacobian^t x p
      else ! if ( work .eq. 't' .or. work .eq. 'T' ) then
          call multphi(1,pp,tmp)
          call multpsi(0,tmp,q)

          do i = 1,nsign
              q(i + nsign) = - q(i)
          end do
      end if

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalhl(n,x,m,lambda,scalef,scalec,hllin,hlcol,hlval,
     +hlnnz,flag)

      implicit none

C     SCALAR ARGUMENTS
      integer flag,hlnnz,m,n
      double precision scalef

C     ARRAY ARGUMENTS
      integer hlcol(*),hllin(*)
      double precision hlval(*),lambda(m),scalec(m),x(n)

      flag = - 1

      end

C     ******************************************************************
C     ******************************************************************

      subroutine evalhlp(n,x,m,lambda,sf,sc,p,hp,goth,flag)

      implicit none

C     SCALAR ARGUMENTS
      logical goth
      integer flag,m,n
      double precision sf

C     ARRAY ARGUMENTS
      double precision hp(n),lambda(m),p(n),sc(m),x(n)

C     LOCAL SCALARS
      integer i

      flag = 0

      do i = 1,n
          hp(i) = 0.0d0
      end do

      end

C     ******************************************************************
C     ******************************************************************

      subroutine endp(n,x,l,u,m,lambda,equatn,linear)

      implicit none

C     SCALAR ARGUMENTS
      integer m,n

C     ARRAY ARGUMENTS
      logical equatn(m),linear(m)
      double precision l(n),lambda(m),u(n),x(n)

#include "cs.com"

C     LOCAL SCALARS
      integer i,j,sl0norm
      double precision sl1norm

C     LOCAL ARRAYS
      integer signali(nsignmax)
      double precision signal(nsignmax),uv(nsignmax)

      sl0norm = 0
      sl1norm = 0.0d0
      do i = 1,nsign
          uv(i) = x(i) - x(i + nsign)
          if ( uv(i) .ne. 0.0d0 ) then
              sl0norm = sl0norm + 1
          end if
          sl1norm = sl1norm + abs( uv(i) )
      end do

      write(*,*) 'Solution l0-norm =',float(sl0norm) / nsign
      write(*,*) 'Solution l1-norm =',sl1norm

      call multpsi(1,uv,signal)

      do i = 1,nsign
          signali(i) = 
     +    nint( max( 0.0d0, min( 255.0d0 * signal(i), 255.0d0 ) ) )
      end do

      open(10,file='solution.pgm')

      write(10,'(A/)') 'P2'
      write(10,*) '#'
      write(10,*) nsigncols,nsignrows
      write(10,*) '255'

      do i = 1,nsignrows
          write(10,*) (signali((i-1)*nsigncols+j),j=1,nsigncols)
      end do

      close(10)

      end

C     ******************************************************************
C     ******************************************************************

      subroutine multpsi(work,z,w)

      implicit none 

C     SCALAR ARGUMENTS
      integer work

C     ARRAY ARGUMENTS
      double precision w(*),z(*)

C     Psi in R^{nsign x nsign} is such that the true signal can be 
C     represented as a 'sparse' combination of its ROWS.

C     Returns w = Psi z if work=0 and w = Psi^T z if work=1.

#include "cs.com"

C     LOCAL SCALARS
      integer i,j,k
      double precision cte

C     LOCAL ARRAYS
      double precision tmp(nsignmax)

      cte = 1.0d0 / sqrt( 2.0d0 )

      do i = 1,nsign
          w(i) = z(i)
      end do

      if ( work .eq. 0 ) then 
          do i = l2nsign,1,-1
              do j = 1,2 ** ( i - 1 )
                  tmp(j)          = cte * ( w(2 * j - 1) + w(2 * j) )
                  tmp(2**(i-1)+j) = cte * ( w(2 * j - 1) - w(2 * j) )
              end do

              do k = 1,2 ** i
                  w(k) = tmp(k)
              end do
          end do

      else
          do i = 1,l2nsign 
              do j = 1,2 ** ( i - 1 )
                  tmp(2*j-1) = cte * ( w(j) + w(2 ** ( i - 1 ) + j ) )
                  tmp(2*j)   = cte * ( w(j) - w(2 ** ( i - 1 ) + j ) )
              end do

              do k = 1,2 ** i
                  w(k) = tmp(k)
              end do
          end do
      end if

      end

C     ******************************************************************
C     ******************************************************************

      subroutine dswap(a,b)

      implicit none

C     SCALAR ARGUMENTS
      double precision a,b

C     LOCAL SCALARS
      double precision tmp

      tmp = a
      a   = b
      b   = tmp

      end

C     ******************************************************************
C     ******************************************************************

      subroutine perm(work,pcol,z,w)

      implicit none

C     SCALAR ARGUMENTS
      integer work

C     ARRAY ARGUMENTS
      integer pcol(*)
      double precision z(*),w(*)

C     Returns w = P z if work = 0 and w = P^T z if work = 1.

#include "cs.com"

C     LOCAL SCALARS
      integer i

      do i = 1,nsign
          w(i) = z(i)
      end do

      if ( work .eq. 0 ) then
          do i = nsign,1,-1
              call dswap(w(i),w(pcol(i)))
          end do

      else
          do i = 1,nsign
              call dswap(w(i),w(pcol(i)))
          end do
      end if

      end

C     ******************************************************************
C     ******************************************************************

      subroutine multphi(work,z,w)

      implicit none 

C     SCALAR ARGUMENTS
      integer work

C     ARRAY ARGUMENTS
      double precision w(*),z(*)

C     Phi in R^{msampl x nsign} represents the sampling matrix.

C     Returns w = Phi z if work=0 and w = Phi^T z if work=1.

#include "cs.com"

C     LOCAL SCALARS
      integer i,ibstart,j,jstart,k,nbrows,nones,quo,quob,res,resb

C     LOCAL ARRAYS
      integer pcol(nsignmax)
      double precision tmp(nsignmax),tmp2(nsignmax)

      if ( work .eq. 0 ) then

          quo = msampl / lPhi
          res = msampl - quo * lPhi

          ibstart = 1
          do k = 1,lPhi

              if ( k .le. res ) then
                  nbrows = quo + 1
              else
                  nbrows = quo
              end if
              do i = 1,nsign
                  pcol(i) = P(i,k)
              end do
              call perm(0,pcol,z,tmp)
              quob = nsign / nbrows
              resb = nsign - quob * nbrows
              jstart = 1

              do i = 1,nbrows
                  if ( i .le. resb ) then
                      nones = quob + 1
                  else
                      nones = quob
                  end if
                  w(ibstart + i - 1) = 0.0d0
                  do j = jstart,jstart + nones - 1
                      w(ibstart + i - 1) = w(ibstart + i - 1) + tmp(j)
                  end do

                  w(ibstart + i - 1) =  w(ibstart + i - 1) 
     +                               / sqrt( 1.0d0 * nones )
                  jstart = jstart + nones
              end do
              ibstart = ibstart + nbrows
              
          end do
      else
          do i = 1,nsign
              w(i) = 0.0d0
          end do

          quo = msampl / lPhi
          res = msampl - quo * lPhi

          ibstart = 1
          do k = 1,lPhi
              if ( k .le. res ) then
                  nbrows = quo + 1
              else
                  nbrows = quo
              end if

              quob = nsign / nbrows
              resb = nsign - quob * nbrows

              jstart = 1
              do i = 1,nbrows
                  if ( i .le. resb ) then
                      nones = quob + 1
                  else
                      nones = quob
                  end if

                  do j = jstart,jstart + nones - 1
                      tmp(j) = z(ibstart + i - 1) 
     +                       / sqrt( 1.0d0 * nones )
                  end do

                  jstart = jstart + nones
              end do

              do i = 1,nsign
                  pcol(i) = P(i,k)
              end do
              
              call perm(1,pcol,tmp,tmp2)

              do i = 1,nsign
                  w(i) = w(i) + tmp2(i)
              end do

              ibstart = ibstart + nbrows
          end do
      end if

      end

C     ******************************************************************
C     ******************************************************************

      subroutine CGrad(w)
      implicit none

C     ARRAYS PARAMETER      
      double precision w(*)

#include "cs.com"

C     LOCAL VARIABLES
      double precision  B(msamplmax), r(msamplmax), q(msamplmax),
     +       d(msamplmax), tmp(msamplmax), tmp2(msamplmax), 
     +       x(msamplmax), xold(msamplmax)
      integer i, imax, j, linf2, linf3
      double precision  alphaCG, deltaNew, deltaOld, DeltaNougt, BetaCG,
     +                  pdq, linf, deltaNew2

      do i = 1, msampl
          x(i) = 0.0d0
      end do

C     !Initial Residual
C     !   r = b - A * x
      call multphi(1, x, tmp)
      call multphi(0, tmp, tmp2)
      do i = 1, msampl
           r(i) = y(i) - tmp2(i)
      end do
      linf = 0.0d0
      linf2 = 0
      do i = 1,  msampl
           linf =max(linf, abs(y(i)))
           if (y(i).ne.0.0d0) then
              linf2 = linf2 + 1
           end if
      end do
      write(*,*) linf, linf2

C      r = B - matmul(A,x)

C     ! Initial Iteration
      deltaNew = 0.0d0
      deltaNew2 = 0.0d0
      do i = 1, msampl
          d(i) = r(i)
          deltaNew = deltaNew + r(i) ** 2
          deltaNew2 = max(deltaNew2, abs(r(i)))
      end do
C      d = r
C      deltaNew = sum(r * r)

C     Iteration Control
      imax = 100

C     Iteration
      do i = 1, imax

C        q = A * d
         call multphi(1, d, tmp)
         call multphi(0, tmp, q)
C         q = matmul(A,d)

C        Line search along residual
         pdq = 0.0d0
         do  j = 1, msampl
              pdq = pdq + d(j) * q(j)
         end do
         alphaCG = deltaNew / pdq
C         alphaCG = deltaNew / sum(d * q)

C        Update Coefficients
         linf3 = 0
         do j  = 1, msampl
             xold(j) = x(j)
             x(j)    = x(j) + alphaCG * d(j)
             if ( abs(x(j)).gt.1.0d-08) then 
                 linf3 = linf3  + 1
             end if
         end do
C         x = x +  alphaCG * d

C        ! Residual
         do j = 1, msampl
             r(j) = r(j) - alphaCG * q(j)
         end do
C         r = r - alphaCG * q

         DeltaOld = DeltaNew
         DeltaNew = 0.0d0
         DeltaNew2 = 0.0d0
         do j = 1, msampl
              DeltaNew = DeltaNew + r(j) ** 2
              DeltaNew2 = max(DeltaNew2, abs(r(j)))
         end do
C         DeltaNew = sum(r * r)
         BetaCG = DeltaNew/DeltaOld
         do j = 1, msampl
             d(j) = r(j) + BetaCG * d(j)
         end do
C         d = r + BetaCG * d

C         Write
         write(*,*) i, DeltaNew, alphaCG, deltaNew2, linf3
C         Exit Criteria

         if(Deltanew < epsilon(1.0)**2.or.DeltaNew.gt.DeltaOld)  exit
C         if(Deltanew < epsilon(1.0)) exit
      end do
      do i = 1, msampl
           w(i) = x(i)
      end do
C     write(*,*) "x=", x
      end

